# Standard Library Imports
from typing import Any, Dict

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseView

class RAMSTKPoFView(RAMSTKBaseView):
    _root: int = ...
    _tag: str = ...
    _dic_load_functions: Any = ...
    _dic_trees: Any = ...
    _lst_modules: Any = ...
    def __init__(self, **kwargs: Dict[Any, Any]) -> None: ...
    def _do_load_mechanisms(self) -> None: ...
    def _do_load_oploads(self, mechanism_id: int) -> None: ...
    def _do_load_opstress(self, load_id: int, parent_id: str) -> None: ...
    def _do_load_test_method(self, load_id: int, parent_id: str) -> None: ...
