# Standard Library Imports
from typing import Any, Tuple

def get_hazard_rate(scale: float, location: float = ...) -> float: ...
def get_mtbf(rate: float, location: float = ...) -> float: ...
def get_survival(scale: float, time: float, location: float = ...) -> float: ...
def do_fit(data: Any, **kwargs: Any) -> Tuple[float, float]: ...
