# Standard Library Imports
from typing import Any, Tuple

def get_hazard_rate(
    shape: float, time: float, location: float = ..., scale: float = ...
) -> float: ...
def get_mtbf(shape: float, location: float = ..., scale: float = ...) -> float: ...
def get_survival(
    shape: float, time: float, location: float = ..., scale: float = ...
) -> float: ...
def do_fit(data: Any, **kwargs: Any) -> Tuple[float, float, float]: ...
