# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.views.gtk3 import GObject as GObject
from ramstk.views.gtk3 import _ as _

class RAMSTKWidget:
    _default_height: int = ...
    _default_width: int = ...
    dic_handler_id: Any = ...
    height: int = ...
    width: int = ...
    def __init__(self) -> None: ...
    def do_set_properties(self, **kwargs: Any) -> None: ...
