# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKActionRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any = ...
    __tablename__: str = ...
    __table_args__: Any = ...
    revision_id: Any = ...
    hardware_id: Any = ...
    mode_id: Any = ...
    mechanism_id: Any = ...
    cause_id: Any = ...
    action_id: Any = ...
    action_recommended: Any = ...
    action_category: Any = ...
    action_owner: Any = ...
    action_due_date: Any = ...
    action_status: Any = ...
    action_taken: Any = ...
    action_approved: Any = ...
    action_approve_date: Any = ...
    action_closed: Any = ...
    action_close_date: Any = ...
    cause: Any = ...
    is_mode: bool = ...
    is_mechanism: bool = ...
    is_cause: bool = ...
    is_control: bool = ...
    is_action: bool = ...
    def get_attributes(self): ...
