# Standard Library Imports
from typing import Any, Tuple

def get_hazard_rate(location: float, scale: float, time: float) -> float: ...
def get_mtbf(location: float, scale: float) -> float: ...
def get_survival(location: float, scale: float, time: float) -> float: ...
def do_fit(data: Any, **kwargs: Any) -> Tuple[float, float]: ...
