# pylint: disable=unused-import
# -*- coding: utf-8 -*-
#
#       ramstk.models.design_mechanic.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK Design Mechanic model package."""
